//+------------------------------------------------------------------+
//|                                          (2009.07.09) CCI_HP.mq4 |
//|                                                         laro_ron |
//|                                                                  |
//|           CCI smoothed by Hodrick-Prescott Filter                |
//|                                                                  |
//|http://en.wikipedia.org/wiki/Hodrick-Prescott_Filter              |
//|The Hodrick-Prescott filter is a mathematical tool used in        |
//|macroeconomics, especially in real business cycle theory. It is   |
//|used to obtain a smoothed non-linear representation of            |
//|a time series, one that is more sensitive to long-term than to    |
//|short-term fluctuations.                                          |
//+------------------------------------------------------------------+

#property copyright "laro_ron"
#property link      "29.09.ron@gmail.com"
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Magenta
#property indicator_width1 1
#property indicator_color2 Khaki
#property indicator_style2  STYLE_DOT
#include <tmplib1.mqh>

//Global constants
#define pi 3.141592653589793238462643383279502884197169399375105820974944592

//Input parameters
extern int LengthCCI =14;               // Number of bars for CCI
extern int AppliedPrice =5;             // Number of Applied Price
extern string SmoothHP="Hodrick-Prescott Filter";
extern int LastVisible    =150;         // Number of bars to smooth for last bar - visible part
extern int LengthHP =70;                // Number of filtered bars (equivalent to SMA period)


double lambda;       // constant as function of Length
int NBEP;            // Number of bars to smooth for End Point
int drop;            // start drawing

//Indicator buffers
double hpf[];        // buffer HP in Last Bar (LB)
double cci_hpf[];    // buffer CCI in the End Points (EP)

int init()
{
   if(LengthHP<2){
      LengthHP=2;
      LastVisible=0;
   }      
   
   
   lambda=0.0625/MathPow(MathSin(pi/LengthHP),4);      // constant
   NBEP=LengthHP*1.5;
   drop=LengthHP*2;

// Buffer for smoothed CCI   
   SetIndexBuffer(0,cci_hpf);     // buffer HP in the EP
   SetIndexDrawBegin(0,drop);     // start drawing from the data beginning 
   SetIndexLabel(0,"CCI("+LengthCCI+")");
   
// Buffer for redrawing HPF of smoothed CCI in the End Point   
   SetIndexBuffer(1,hpf);         // buffer HP in LB
   SetIndexDrawBegin(1,drop);     // start drawing from the data beginning
   SetIndexLabel(1,"Smooth HP("+LengthHP+")");

   
   IndicatorShortName("CCI_HP("+LengthCCI+","+LengthHP+")");   //name in DataWindow

   return(0);
}
//+----------------------------------------------------------------------------------------+
int start()
{
   int repainted_bars;           // the number of changed bars
   int counted_bars;             // the number of bars not changed after the indicator had been launched last
   
   double x[];                   // predictors in last bar
   ArrayResize(x,LastVisible);

   double x_[];               // predictors end points
   ArrayResize(x_,NBEP);
   double y_[];               // responses end points
   ArrayResize(y_,NBEP);
   
   counted_bars=IndicatorCounted();         //not changed Bars
   if(counted_bars>0) counted_bars--;       //last counted bar will be recounted
   repainted_bars=Bars-counted_bars;                 //number bars from the chart beeing repainted
   
   Comment("Bars : ",Bars,
           "\nRepainted : ",repainted_bars,
           "\nTime : ",TimeToStr(TimeCurrent(),TIME_DATE|TIME_SECONDS));
   
   
   for (int j=0;j<repainted_bars;j++){
      for(int i=0;i<NBEP;i++) x_[i]=iCCI(NULL,0,LengthCCI,AppliedPrice,j+i);       // fill predictors x_[]
      cci_hpf[j]=HPF(NBEP,lambda,x_,y_);                 // calculation cci_hpf[]   NBEP = number counted for End Point
   }
  
// Hodrick-Prescott Filter in Last Bar for last visible part 

   for(i=0;i<LastVisible;i++) x[i]=iCCI(NULL,0,LengthCCI,AppliedPrice,i);    // fill predictors x[]
   HPF(LastVisible,lambda,x,hpf);                // calculation hpf[]   LastVisible = number counted
   
//----------------------------------------------------
   return(0);
}


