//+------------------------------------------------------------------+
//|                                                       DMI_HP.mq4 |
//|                                                         laro_ron |
//|  snapshot without After Smooth                                   |
//+------------------------------------------------------------------+
#property copyright "laro_ron"
#property link      "29.09.ron@gmail.com"
//----
#property indicator_separate_window
#property indicator_buffers   4
#property indicator_color1    LightBlue
#property indicator_width1    1
#property indicator_color2    Lime
#property indicator_width2    1
#property indicator_color3    Tomato
#property indicator_width3    1
#property indicator_color4    Orange
#property indicator_width4    1
#property indicator_level1    20

// Libraries
#include <tmplib1.mqh>

//Global constants
#define pi 3.141592653589793238462643383279502884197169399375105820974944592

//---- input parameters
extern int    Smooth_Length  =10;      // Period of additional smoothing 
extern string Smooth_Type_Info =  "-1 Nothing   0 EMA   1 HMA   2 HP";
extern int    Smooth_Type = 1;         // Type of smoothing Price -1-nothing, 0-EMA, 1-HMA, 2-HP, 3-LinReg, 4-QuadReg, 5-LSPF3
extern int    DMI_Length  = 14;        // Period of DMI
extern int    ADX_Length  = 14;        // Period of ADX
extern int    ADXR_Length = 14;        // Period of ADXR
extern bool   UseADX      = true;      // True - Use ADX
extern bool   UseADXR     = false;      // True - Use ADXR
extern bool   DrawDots    = true;     // True - dots
//extern string After_Smooth_Type_Info =  "-1 Nothing   0 SMA   1 HMA   2 HP";
//extern int    After_Type  = 1;      // HMA
//extern int    After_DMI_Length = 5;    // After smoothing DMI
//extern int    After_ADX_Length = 1;    // After smoothing ADX
//extern int    After_ADXR_Length = 3;    // After smoothing ADXR

//---- buffers
double ADX[];
double PDI[];
double MDI[];
double ADXR[];
double sPDI[];
double sMDI[];
double STR[];
double DX[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+

int init()
  {
//---- indicators
   IndicatorBuffers(8);
   SetIndexBuffer(0,ADX);
   SetIndexBuffer(1,PDI);
   SetIndexBuffer(2,MDI);
   SetIndexBuffer(3,ADXR);
   SetIndexBuffer(4,sPDI);
   SetIndexBuffer(5,sMDI);
   SetIndexBuffer(6,STR);
   SetIndexBuffer(7,DX);
   if (DrawDots) {
      SetIndexStyle(0,DRAW_ARROW);
      SetIndexStyle(1,DRAW_ARROW);
      SetIndexStyle(2,DRAW_ARROW);
      SetIndexStyle(3,DRAW_ARROW);
      SetIndexArrow(0,158);
      SetIndexArrow(1,158);
      SetIndexArrow(2,158);
      SetIndexArrow(3,158);
   }
   else {
      SetIndexStyle(0,DRAW_LINE);
      SetIndexStyle(1,DRAW_LINE);
      SetIndexStyle(2,DRAW_LINE);
      SetIndexStyle(3,DRAW_LINE);
   }

     
//---- name for DataWindow and indicator subwindow label
   string short_name="DMI_HP("+Smooth_Length+","+DMI_Length+","+ADX_Length+","+ADXR_Length+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"ADX("+ADX_Length+")");
   SetIndexLabel(1,"+DI("+DMI_Length+")");
   SetIndexLabel(2,"-DI("+DMI_Length+")");
   SetIndexLabel(3,"ADXR("+ADXR_Length+")");
//----
   SetIndexDrawBegin(0,DMI_Length+Smooth_Length);
   SetIndexDrawBegin(1,DMI_Length+Smooth_Length);
   SetIndexDrawBegin(2,DMI_Length+Smooth_Length);
   SetIndexDrawBegin(3,DMI_Length+Smooth_Length);
   SetIndexShift(3,2);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int shift, limit, counted_bars=IndicatorCounted();
   double alfa1=1.0/DMI_Length;
   double alfa2=1.0/ADX_Length;
//---- 
   if (counted_bars < 0) return(-1);
   if(counted_bars<1)
      for(shift=1;shift<=Smooth_Length+DMI_Length;shift++)
        {
         PDI[Bars-shift]=0.0;MDI[Bars-shift]=0.0;ADX[Bars-shift]=0.0;
         sPDI[Bars-shift]=0.0;sMDI[Bars-shift]=0.0;DX[Bars-shift]=0.0;
         STR[Bars-shift]=0.0;ADXR[Bars-shift]=0.0;
        }
   shift=Bars-Smooth_Length-DMI_Length-1;
   if(counted_bars>=DMI_Length+Smooth_Length) shift=Bars-counted_bars-1;
   while(shift>=0)
     {
      double AvgHigh   = Smooth(Smooth_Type,Smooth_Length,PRICE_HIGH,shift);
      double AvgHigh1  = Smooth(Smooth_Type,Smooth_Length,PRICE_HIGH,shift+1);
      double AvgLow    = Smooth(Smooth_Type,Smooth_Length,PRICE_LOW,shift);
      double AvgLow1   = Smooth(Smooth_Type,Smooth_Length,PRICE_LOW,shift+1);
      double AvgClose1 = Smooth(Smooth_Type,Smooth_Length,PRICE_CLOSE,shift+1);
//----
      double Bulls=0.5*(MathAbs(AvgHigh-AvgHigh1)+(AvgHigh-AvgHigh1));
      double Bears=0.5*(MathAbs(AvgLow1-AvgLow)+(AvgLow1-AvgLow));
      if (Bulls > Bears) Bears=0;
      else
         if (Bulls < Bears) Bulls=0;
         else
            if (Bulls==Bears) {Bulls=0;Bears=0;}
//----            
      sPDI[shift]=sPDI[shift+1] + alfa1 * (Bulls - sPDI[shift+1]);
      sMDI[shift]=sMDI[shift+1] + alfa1 * (Bears - sMDI[shift+1]);
//----      
      double   TR=MathMax(AvgHigh-AvgLow,AvgHigh-AvgClose1);
      STR[shift] =STR[shift+1] + alfa1 * (TR - STR[shift+1]);
      if(STR[shift]>0 )
        {
         PDI[shift]=100*sPDI[shift]/STR[shift];
         MDI[shift]=100*sMDI[shift]/STR[shift];
        }
      if(UseADX)
        {
         if((PDI[shift] + MDI[shift])>0)
            DX[shift]=100*MathAbs(PDI[shift] - MDI[shift])/(PDI[shift] + MDI[shift]);
         else DX[shift]=0;
//----
         ADX[shift]=ADX[shift+1] + alfa2 *(DX[shift] - ADX[shift+1]);
         if(UseADXR)ADXR[shift]=0.5*(ADX[shift] + ADX[shift+ADXR_Length]);
        }



//      if (After_Type>-1) {
//         Smth(After_Type,After_DMI_Length,PDI,shift);                      // PDI   MDI
//         Smth(After_Type,After_DMI_Length,MDI,shift);
//         if (UseADX) {
//            Smth(After_Type,After_ADX_Length,ADX,shift);                   // ADX
//            if (UseADXR) Smth(After_Type,After_ADXR_Length,ADXR,shift);    // ADXR
//         }
//      }


      shift--;
     }
//----
   return(0);
  }
//+------------------------------------------------------------------+


//void Smth(int Type, int Length, double& y[], int i)
//{
//   double x[];
//   ArrayResize(x,Length);
//   for (int j=0; j<Length; j++)  
//      x[j]=y[j+i]; 
//      y[i]=MA_(Length, x,0);
//}


// Smooth Price ----------------------------------------------------------------------+
double Smooth(int type, int length, int price, int i)
{
   double ret;
   switch(type){
      case -1:  ret=GAP(price,i);                          break;
      case 0:  ret=iMA(NULL,0,length,0,MODE_EMA,price,i);  break;
      case 1:  ret=HM(length,price,3,i);                   break;
      case 2:  ret=HP(length,price,i);                     break;    
      default: ret=GAP(price,i);                           break;
   }
   return(ret);
}

/*
MODE_SMA 0 Simple moving average, 
MODE_EMA 1 Exponential moving average, 
MODE_SMMA 2 Smoothed moving average, 
MODE_LWMA 3 Linear weighted moving average. 
*/


double HP(int length, int price, int i)
{
   double lambda=0.0625/MathPow(MathSin(pi/length),4);      // constant
   double x[],y[];
   int k;
   ArrayResize(x,length);
   ArrayResize(y,length);
   for(k=0;k<length;k++) x[k]=GAP(price,k+i);
   return(HPF(length,lambda,x,y));
}

double HM(int length, int price, int method, int i)
{
   double x[];
   int k;
   ArrayResize(x,length);
   for(k=0;k<length;k++) x[k]=GAP(price,k+i);
   return(HMA_(length,x));
}






