//+------------------------------------------------------------------+
//|                                                      tmplib1.mq4 |
//|                                                         laro_ron |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "laro_ron"
#property link      "29.09.ron@gmail.com"
#property library

//+------------------------------------------------------------------+
//| My functions                                                     |
//+------------------------------------------------------------------+

// Get Applied Price ----------------------------------------------------------------------+
double GAP(int nAppliedPrice, int nIndex)
{
   double dPrice;
   switch(nAppliedPrice){
      case 0:  dPrice=Close[nIndex];                                  break;
      case 1:  dPrice=Open[nIndex];                                   break;
      case 2:  dPrice=High[nIndex];                                   break;
      case 3:  dPrice=Low[nIndex];                                    break;
      case 4:  dPrice=(High[nIndex]+Low[nIndex])/2.0;                 break;
      case 5:  dPrice=(High[nIndex]+Low[nIndex]+Close[nIndex])/3.0;   break;
      case 6:  dPrice=(High[nIndex]+Low[nIndex]+2*Close[nIndex])/4.0; break;
      case 7:  if(Close[nIndex]>Close[nIndex+1])
                  {dPrice=High[nIndex]; break;}
               else
                  {dPrice=Low[nIndex];break;}
      default: dPrice=Close[nIndex];
   }
   return(dPrice);
}


// Standard Deviation ----------------------------------------------------------------------+
double StdD(int nb, double x[])
{
   if (nb<1) return(0.0);
   double sum1=0.0;
   double sum2=0.0;
   for (int i=0;i<nb;i++) {
      sum1+=x[i];
      sum2+=x[i]*x[i];
   }
   return(MathSqrt((sum2-MathPow((sum1/nb),2))/nb));
}

// Hodrick-Prescott Filter -----------------------------------------------------------------+
double HPF(int nb, double lmbd, double x[], double& y[])
{
   double a[],b[],c[],H1,H2,H3,H4,H5,HH1,HH2,HH3,HH4,HH5,HB,HC,Z;
   ArrayResize(a,nb);
   ArrayResize(b,nb);
   ArrayResize(c,nb);
        
   a[0]=1.0+lmbd;
   b[0]=-2.0*lmbd;
   c[0]=lmbd;
   for(int i=1;i<nb-2;i++){
      a[i]=6.0*lmbd+1.0;
      b[i]=-4.0*lmbd;
      c[i]=lmbd;
   }
   a[1]=5.0*lmbd+1;
   a[nb-1]=1.0+lmbd;
   a[nb-2]=5.0*lmbd+1.0;
   b[nb-2]=-2.0*lmbd;
   b[nb-1]=0.0;
   c[nb-2]=0.0;
   c[nb-1]=0.0;
   
   //Forward
   for(i=0;i<nb;i++){
      Z=a[i]-H4*H1-HH5*HH2;
      HB=b[i];
      HH1=H1;
      H1=(HB-H4*H2)/Z;
      b[i]=H1;
      HC=c[i];
      HH2=H2;
      H2=HC/Z;
      c[i]=H2;
      a[i]=(x[i]-HH3*HH5-H3*H4)/Z;
      HH3=H3;
      H3=a[i];
      H4=HB-H5*HH1;
      HH5=H5;
      H5=HC;
   }
   
   //Backward 
   H2=0;
   H1=a[nb-1];
   y[nb-1]=H1;
   for(i=nb-2;i>=0;i--){
      y[i]=a[i]-b[i]*H1-c[i]*H2;
      H2=H1;
      H1=y[i];
   }
   return(y[0]);
}


// Hull Moving Average -----------------------------------------------------------------+
double HMA_(int length, double x[])
{
   double buffer[];
   ArrayResize(buffer,length);
   for (int j=0; j<length; j++)  
      buffer[j]=2*MA_(MathFloor(length/2),x,j)-MA_(length,x,j); 
   return(MA_(MathFloor(MathSqrt(length)),buffer,0));
}

double MA_(int nb, double x[], int shift)
{
   if (nb<1) return(0.0);
   double sum1=0.0;
   for (int i=shift;i<shift+nb;i++) sum1+=x[i];
   return(sum1/nb);
}



